//---------------------------------------------------
// module  : f_KmeanConstrain.cc                       
// auteur  : Max Mignotte                            
// date    :                                              
// langage : C++                                      
// labo    :                                  
//---------------------------------------------------

//-----------------------------------------------
// FICHIERS INCLUS ------------------------------
//-----------------------------------------------
#include "KmeanConstrain.h"

//------------------------------------------------
// METHODES DE LA CLASSE KMEANCONSTRAIN  ---------
//------------------------------------------------
//----------------------------------------------------------
// Constructeur de la classe KmeanConstrain
//----------------------------------------------------------
KmeanConstrain::KmeanConstrain(int*** img,int lgth,int wdth)           
 {
  int i,j,k;

  //Initialisation
  length=lgth;
  width=wdth;

  //Allocation Memoire Matrice 
  mat_img=imatrix_allocate_3d(TROIS,length,width);
  mat_seg=fmatrix_allocate_2d(length,width);

  mat_imgout=imatrix_allocate_3d(TROIS,length,width);
  mat_imgouf=fmatrix_allocate_3d(TROIS,length,width);

  smat_seg=fmatrix_allocate_3d(NB_ESPACE_COUL,length,width); 
  tmat_seg=fmatrix_allocate_3d(NB_ESPACE_COUL,length,width);

  mat1=fmatrix_allocate_2d(length,width);

  //Initialisation
  for(k=0;k<TROIS;k++) for(i=0;i<length;i++) for(j=0;j<width;j++) 
  mat_img[k][i][j]=img[k][i][j];  
 }


//----------------------------------------------------------
// Destructeur de la classe KmeanConstrain
//----------------------------------------------------------
KmeanConstrain::~KmeanConstrain()
{ 
  //Libere Memoire Matrice 
  if (mat_img)    free_imatrix_3d(mat_img,TROIS);
  if (mat_seg)    free_fmatrix_2d(mat_seg);

  if (mat_imgout)  free_imatrix_3d(mat_imgout,TROIS);
  if (mat_imgouf)  free_fmatrix_3d(mat_imgouf,TROIS);

  if (smat_seg)   free_fmatrix_3d(smat_seg,NB_ESPACE_COUL); 
  if (tmat_seg)   free_fmatrix_3d(tmat_seg,NB_ESPACE_COUL);

   if (mat1)  free_fmatrix_2d(mat1);
}

//----------------------------
//----<Histo><COLOR-BASE>-----
//----------------------------
//----------------------------------------------------------
//  Segmentation <Histo-COLOR-> Kmean
//       < Vraie Histo >
//
//  image[lgth][wdth] en NbClass 
//  Histo divise en <div> bin
//  SizeWindow: <Size>
// 
//  bascol::
//  -----------            
//  -0- RGB ->    [X,Y,Z] = [0,255]
//  -1- HSV ->    H=[0::360]  S=[0::1]   V=[0::255]
//  -2- YIQ -     Y=[0::1]    I=[-0.55::0.55]  Q=[-0.4::0.4]
//  -3- XYZ ->    [X,Y,Z] = [0,255]
//  -4- LAB -     L=[0::100]    A=[-128::143]  B=[-258::237]
//  -5- LUV -     L=[-16::100]  U=[-60::145]   V=[-130::117]    
//
//  -6- i123-     i1=[0::255]    i2=[-255::255]      i3=[-255::255]  
//  -7- h123-     h1=[0::510]    h2=[-255::255]      h3=[-255::255]
//  -8- YCbCr    Y=[0::255]     Cb=[-127.5::127.5]  Cr=[-127.5::127.5]
//  -9- TSL-      T=[0::1]       S=[0::1.265]        L=[0::1]
//  -10-HSL-      H=[0::3.142]   S=[0::1]            L=[0::1]    
//  -11- P1P2     P1=[-0.707::0.707]   P2=[-0.408::0.816]   
//
//  Recalage sur [0-255] avec
//
//             (C-min)*255/(max-min)
//
//----------------------------------------------------------     
//----------------------------------------------------------     
// SegmentationHistoCOLORKmean Avec Entier 
//----------------------------------------------------------     
void KmeanConstrain::SegmentationHistoCOLORKmeanFast(int***  img,float** matout,int lgth,int wdth,int NbClass,int div,int SzW,int Seed,int bascol)
{
 int i,j,k,l;
 int nbpts; 
 int posr,posc;
 float divis;
 float lab;
 float col1,col2,col3;

 //Init
 int NbParam=div*div*div; 
 divis=(256.0/(float)div);  
 
 //Initialisatio memoire
 int** PtsTab=imatrix_allocate_2d(lgth*wdth,NbParam);
 float*** SquWin=fmatrix_allocate_3d(3,SzW,SzW);
 float*** ImgColorSpace=fmatrix_allocate_3d(3,lgth,wdth);
 int*  TabLabel=imatrix_allocate_1d(NbParam);
 int* VctLabel=imatrix_allocate_1d(lgth*wdth);  

 //Presentation
 printf("\n   >>> SegmentationHisto-COLOR-Kmean");
 printf("[NbCl:%d][Div:%d][Ws:%d][Col:%d][Par:%d]",NbClass,div,SzW,bascol,NbParam); 
 if (bascol==0)  printf(" >RGB");
 if (bascol==1)  printf(" >HSV");
 if (bascol==2)  printf(" >YIQ");
 if (bascol==3)  printf(" >XYZ");
 if (bascol==4)  printf(" >LAB");
 if (bascol==5)  printf(" >LUV");
 if (bascol==6)  printf(" >I1I2I3");
 if (bascol==7)  printf(" >H1H2H3");
 if (bascol==8)  printf(" >YCbCr");
 if (bascol==9)  printf(" >TSL");
 if (bascol==10) printf(" >HSL"); 
 if (bascol==11) printf(" >P1P2"); 
 fflush(stdout);
 

 //-------------------------------------------------------------------------
 //Conversion
 //-------------------------------------------------------------------------
 if (bascol==0)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { ImgColorSpace[0][i][j]=img[0][i][j];
      ImgColorSpace[1][i][j]=img[1][i][j];
      ImgColorSpace[2][i][j]=img[2][i][j]; }

 if (bascol==1)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToHSV(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3); 
      ImgColorSpace[0][i][j]=(col1/360.0)*255.0;
      ImgColorSpace[1][i][j]=col2*255.0;
      ImgColorSpace[2][i][j]=col3; }

 if (bascol==2)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToYIQ(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3); 
      ImgColorSpace[0][i][j]=col1*255.0;
      ImgColorSpace[1][i][j]=((col2+0.55)/1.10)*255.0;
      ImgColorSpace[2][i][j]=((col3+0.4)/0.8)*255.0; }

 if (bascol==3)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToXYZ(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=col1;
      ImgColorSpace[1][i][j]=col2;
      ImgColorSpace[2][i][j]=col3; }

 if (bascol==4)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToLAB(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=col1*2.55;
      ImgColorSpace[1][i][j]=((col2+128)/271.0)*255.0;
      ImgColorSpace[2][i][j]=((col3+258)/495.0)*255.0; }

 if (bascol==5)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
   {  RGBToLUV(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=((col1+16)/250.0)*255.0;
      ImgColorSpace[1][i][j]=((col2+60)/205.0)*255.0;
      ImgColorSpace[2][i][j]=((col3+130)/247.0)*255.0; }

 if (bascol==6)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToI1I2I3(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=col1;
      ImgColorSpace[1][i][j]=((col2+255)/2.0);
      ImgColorSpace[2][i][j]=((col3+255)/2.0);  }

 if (bascol==7)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToH1H2H3(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=col1/2.0; 
      ImgColorSpace[1][i][j]=((col2+255)/2.0); 
      ImgColorSpace[2][i][j]=((col3+255)/2.0); }

 if (bascol==8)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToYCbCr(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=col1; 
      ImgColorSpace[1][i][j]=col2+127.5; 
      ImgColorSpace[2][i][j]=col3+127.5; }

 if (bascol==9)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToTSL(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=col1*255;
      ImgColorSpace[1][i][j]=255*col2/1.265; 
      ImgColorSpace[2][i][j]=col3*255;  }

 if (bascol==10)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToHSL(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=255*col1/3.142;  
      ImgColorSpace[1][i][j]=col2*254;
      ImgColorSpace[2][i][j]=col3*255; }

 if (bascol==11)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToP1P2(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[0][i][j]=(col1+0.707)*255.0/1.414; 
      ImgColorSpace[1][i][j]=(col2+0.408)*255.0/1.224; 
      ImgColorSpace[2][i][j]=0.0; }


 //Debug
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
   { if (ImgColorSpace[0][i][j]>255)  ImgColorSpace[0][i][j]=255.0;
     if (ImgColorSpace[1][i][j]>255)  ImgColorSpace[1][i][j]=255.0;
     if (ImgColorSpace[2][i][j]>255)  ImgColorSpace[2][i][j]=255.0;
     if (ImgColorSpace[0][i][j]<0)    ImgColorSpace[0][i][j]=0.0;
     if (ImgColorSpace[1][i][j]<0)    ImgColorSpace[1][i][j]=0.0;
     if (ImgColorSpace[2][i][j]<0)    ImgColorSpace[2][i][j]=0.0; }
 //-------------------------------------------------------------------------- 
  

 //---------------------------------------------------------------
 //Data   
 //----------------------------------------------------------------                       
 nbpts=0;
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++)  
    {
     for(k=0;k<SzW;k++) for(l=0;l<SzW;l++) 
        { 
	 posr=i-(SzW/2)+k;
         posc=j-(SzW/2)+l;

         if (posr<0)           posr+=lgth;
         if (posr>(lgth-1))    posr-=lgth;  
         if (posc<0)           posc+=wdth;
         if (posc>(wdth-1))    posc-=wdth;
        
         SquWin[0][k][l]=ImgColorSpace[0][posr][posc];
         SquWin[1][k][l]=ImgColorSpace[1][posr][posc];
         SquWin[2][k][l]=ImgColorSpace[2][posr][posc];  
        }

     //DataStructure
     for(k=0;k<NbParam;k++)  TabLabel[k]=0;
      
     for(k=0;k<SzW;k++) for(l=0;l<SzW;l++)  
	{ lab=((int)(SquWin[0][k][l]/divis))*(div*div);
          lab+=((int)(SquWin[1][k][l]/divis))*(div);  
          lab+=((int)(SquWin[2][k][l]/divis));  
          TabLabel[(int)(lab)]++; }
    
     for(k=0;k<NbParam;k++)    PtsTab[nbpts][k]=TabLabel[k];

     nbpts++;     
    }//----------------------------------------------------------------

 //K-Mean    
 KmeanFast(PtsTab,NbClass,(lgth*wdth),NbParam,VctLabel,Seed,1);
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++)  matout[i][j]=VctLabel[(i*wdth)+j];    

 //Liberation Memoire
 if (PtsTab)         free_imatrix_2d(PtsTab);
 if (SquWin)         free_fmatrix_3d(SquWin,3);
 if (TabLabel)       free_imatrix_1d(TabLabel);
 if (VctLabel)       free_imatrix_1d(VctLabel);
 if (ImgColorSpace)  free_fmatrix_3d(ImgColorSpace,3);
}

//----------------------------------------------------------     
// SegmentationCOLORKmean Avec Entier 
//----------------------------------------------------------     
void KmeanConstrain::SegmentationMCOLORKmeanFast(int***  img,float** matout,int lgth,int wdth,int NbClass,\
					     int SzW,int Seed,int bascol,float** Pot,float thresh)
{
  int i,j,k,l,m;
 int nbpts; 
 int posr,posc;
 float col1,col2,col3;
 int nb;

 //Init
 int NbParam=10*3*SzW*SzW;   
 
 //Initialisatio memoire
 int** PtsTab=imatrix_allocate_2d(lgth*wdth,NbParam);
 float**** SquWin=fmatrix_allocate_4d(10,TROIS,SzW,SzW);
 float**** ImgColorSpace=fmatrix_allocate_4d(10,TROIS,lgth,wdth);
 int*  TabLabel=imatrix_allocate_1d(NbParam);
 int* VctLabel=imatrix_allocate_1d(lgth*wdth);  

 //Presentation
 printf("\n   >>> Segmentation-M-COLOR-Kmean <Constraint>");
 printf("[NbCl:%d][Ws:%d][Col:%d][Par:%d]",NbClass,SzW,bascol,NbParam); 
 fflush(stdout);
 
 //-------------------------------------------------------------------------
 //Conversion
 //-------------------------------------------------------------------------
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { ImgColorSpace[0][0][i][j]=img[0][i][j];
      ImgColorSpace[0][1][i][j]=img[1][i][j];
      ImgColorSpace[0][2][i][j]=img[2][i][j]; }

 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToHSV(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3); 
      ImgColorSpace[1][0][i][j]=(col1/360.0)*255.0;
      ImgColorSpace[1][1][i][j]=col2*255.0;
      ImgColorSpace[1][2][i][j]=col3; }

 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToYIQ(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3); 
      ImgColorSpace[2][0][i][j]=col1*255.0;
      ImgColorSpace[2][1][i][j]=((col2+0.55)/1.10)*255.0;
      ImgColorSpace[2][2][i][j]=((col3+0.4)/0.8)*255.0; }

 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToXYZ(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[3][0][i][j]=col1;
      ImgColorSpace[3][1][i][j]=col2;
      ImgColorSpace[3][2][i][j]=col3; }

 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToLAB(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[4][0][i][j]=col1*2.55;
      ImgColorSpace[4][1][i][j]=((col2+128)/271.0)*255.0;
      ImgColorSpace[4][2][i][j]=((col3+258)/495.0)*255.0; }

 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
   {  RGBToLUV(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[5][0][i][j]=((col1+16)/250.0)*255.0;
      ImgColorSpace[5][1][i][j]=((col2+60)/205.0)*255.0;
      ImgColorSpace[5][2][i][j]=((col3+130)/247.0)*255.0; }
 
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToI1I2I3(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[6][0][i][j]=col1;
      ImgColorSpace[6][1][i][j]=((col2+255)/2.0);
      ImgColorSpace[6][2][i][j]=((col3+255)/2.0);  }
 
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToH1H2H3(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[7][0][i][j]=col1/2.0; 
      ImgColorSpace[7][1][i][j]=((col2+255)/2.0); 
      ImgColorSpace[7][2][i][j]=((col3+255)/2.0); }

 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToYCbCr(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[8][0][i][j]=col1; 
      ImgColorSpace[8][1][i][j]=col2+127.5; 
      ImgColorSpace[8][2][i][j]=col3+127.5; }

 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
    { RGBToTSL(img[0][i][j],img[1][i][j],img[2][i][j],&col1,&col2,&col3);
      ImgColorSpace[9][0][i][j]=col1*255;
      ImgColorSpace[9][1][i][j]=255*col2/1.265; 
      ImgColorSpace[9][2][i][j]=col3*255;  }

 //Debug
 for(m=0;m<10;m++)
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++) 
   { if (ImgColorSpace[m][0][i][j]>255)  ImgColorSpace[m][0][i][j]=255.0;
     if (ImgColorSpace[m][1][i][j]>255)  ImgColorSpace[m][1][i][j]=255.0;
     if (ImgColorSpace[m][2][i][j]>255)  ImgColorSpace[m][2][i][j]=255.0;
     if (ImgColorSpace[m][0][i][j]<0)    ImgColorSpace[m][0][i][j]=0.0;
     if (ImgColorSpace[m][1][i][j]<0)    ImgColorSpace[m][1][i][j]=0.0;
     if (ImgColorSpace[m][2][i][j]<0)    ImgColorSpace[m][2][i][j]=0.0; }
 //-------------------------------------------------------------------------- 
  

 //---------------------------------------------------------------
 //Data   
 //----------------------------------------------------------------                       
 nbpts=0;
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++)  
    {
     for(k=0;k<SzW;k++) for(l=0;l<SzW;l++) 
        { 
	 posr=i-(SzW/2)+k;
         posc=j-(SzW/2)+l;

         if (posr<0)           posr+=lgth;
         if (posr>(lgth-1))    posr-=lgth;  
         if (posc<0)           posc+=wdth;
         if (posc>(wdth-1))    posc-=wdth;
        
         for(m=0;m<10;m++)
	  { SquWin[m][0][k][l]=ImgColorSpace[m][0][posr][posc];
            SquWin[m][1][k][l]=ImgColorSpace[m][1][posr][posc];
            SquWin[m][2][k][l]=ImgColorSpace[m][2][posr][posc]; }
        }

     //DataStructure
     for(k=0;k<NbParam;k++)  TabLabel[k]=0;
      
     nb=0;
     for(m=0;m<10;m++)
     for(k=0;k<SzW;k++) for(l=0;l<SzW;l++)  
       { TabLabel[(3*nb)+0]=(int)SquWin[m][0][k][l];
         TabLabel[(3*nb)+1]=(int)SquWin[m][1][k][l];
         TabLabel[(3*nb)+2]=(int)SquWin[m][2][k][l]; 
         nb++; }
    
     for(k=0;k<NbParam;k++)    PtsTab[nbpts][k]=TabLabel[k];

     nbpts++;     
    }//----------------------------------------------------------------

 //K-Mean    
 KmeanFast(PtsTab,NbClass,(lgth*wdth),NbParam,VctLabel,Seed,1,Pot,lgth,wdth,thresh);  
 for(i=0;i<lgth;i++) for(j=0;j<wdth;j++)  matout[i][j]=VctLabel[(i*wdth)+j];    

 //Liberation Memoire
 if (PtsTab)         free_imatrix_2d(PtsTab);
 if (SquWin)         free_fmatrix_4d(SquWin,10,TROIS);
 if (TabLabel)       free_imatrix_1d(TabLabel);
 if (VctLabel)       free_imatrix_1d(VctLabel);
 if (ImgColorSpace)  free_fmatrix_4d(ImgColorSpace,10,3);
}

//-------------------------//
//--METHODES SEGMENTATION--//
//-------------------------//
//----------------------------------------------------------
// Segmente <DETEXTURING>
//
//----------------------------------------------------------
void KmeanConstrain::Segmente()
{
  int k;

 //Pour Tests
 //----------
 char MyBuff[200];

 //Record
 //------
 strcpy(MyBuff,Name_Img);
 strcat(MyBuff,Extension);

 //Constantes
 //----------
 const int NBSEGS=10;

 //Allocation Memoire
 //------------------
 SetRegion** tabsetreg=new SetRegion*[NBSEGS*2];

  //Information
 //-----------
 printf("\n ---------------------------------------------------------- ");
 printf("\n Info : MultSpaceColKmeans + DeTEXturing/Estimation ImgCoul ");
 printf("\n ---------------------------------------------------------- ");
 printf("\n");
 printf("\n ---------------------");
 printf("\n               >> NbClass      [%d]",NbCl);
 printf("\n               >> Regul        [%.3f]",Regul);
 //printf("\n               >> Size         [%d]",Size);
 printf("\n ---------------------");
 printf("\n Extension     >> [%s]",Extension);
 printf("\n Image         >> [%s]  Size:[%d - %d]",MyBuff,length,width);
 printf("\n ---------------------");
 printf("\n");
 fflush(stdout);

 //============================================================================
 //>     Kmean :: [NbClass][div][SzW][Seed][bascol] > smat_seg[k]**
 //>                                 DeTexTuring    > tabmat_seg[k]***
 //============================================================================
 for(k=0;k<NBSEGS;k++)
    { printf("\n >>>>[%d]",k);
      SegmentationHistoCOLORKmeanFast(mat_img,smat_seg[k],length,width,5,5,7,k,k);
      ComputeContour(tmat_seg[k],smat_seg[k],length,width); }

 for(k=0;k<NBSEGS;k++)
   { tabsetreg[k]=new SetRegion(smat_seg[k],length,width,5000);
     (*tabsetreg[k]).flag_ssilent=1;
     (*tabsetreg[k]).find_regions();
     (*tabsetreg[k]).stat_regions(); }

 ComputeMatSegMoy(mat_imgouf,mat_img,tabsetreg,NBSEGS,length,width);
 copy_mat(mat_imgouf,mat_imgout,length,width,TROIS);

 //=============================================================
 //> Compute SOFT CONTOURS
 //              [int dist,int div,int SzW]
 //=============================================================
 GradMCS2(mat_img,mat1,length,width,3,5,7);

 //================================================================================
 //SEGMENTATION FINALE
 //         >>> [NbClass,SzW,Seed,bascol,  Pot,thresh]
 //================================================================================
 SegmentationMCOLORKmeanFast(mat_imgout,mat_seg,length,width,NbCl,5,0,0,mat1,Regul);  //Regul=0.37 

 //=============================================================
 //> NETTOYAGE > Fuse small Regions
 //=============================================================
 for(k=0;k<10;k++) if (!FuseSmallRegions(mat_seg,length,width,&NbRegions,Size,0.32))  break;
 printf("\n >>>>>> NbRegions [%d]",NbRegions);
 for(k=0;k<2;k++) MedianFilter(mat_seg,length,width,5); 

 //AddCOntour
 copy_mat(mat_img,mat_imgout,length,width,TROIS);
 AddContourImg(mat_seg,mat_imgout,length,width,3);
 //SaveImagePpm(MyBuff,"CONT",mat_imgout,length,width);

 //Convert Class-->Regions
 ConvertClassRegion(mat_seg,length,width);

 //Sauvegarde
 SaveImagePgm(MyBuff,(char*)"",mat_seg,length,width);
}
